%%This script is for calculating and displaying the I-V characteristics of
%%PV modules under shaded conditions, specified by the user.

%% SIMPLIFICATIONS
%Saturation current I_0 is constant (not varying with voltage)
%Ideality factor A is cosntant (not varying with voltage)
%R_s is sufficiently small so that I_sc can be assumed I_ph
%Parasittic resistiances are constand, even though they're not really.

clear all
clc

%%
e=1;
while e~=0 %Input window stays open until values are OK.
prompt = {'Number of cells in the module (rows x columns)', ... %input 1
    'Number of bypass diodes/substrings:', ... %input 2
    'Rated short-circuit current (cell), STC [A]:',... %input 3
    'Rated open-circuit voltage (module), STC [V]:', ... %input 4
    'Cell parallel resistance [ohm]:', ... %input 5
    'Cell series resistance [ohm]:', ... %input 6
    'Diode ideality factor:', ... %input 7
    'Diode reverse saturation current at 25 deg C [A]:', ... %input 8
    'Temperature [degrees Celcius]:', ... %input 9
    'Step size for calculation increment [A]:'}; %input 10

options.Resize='on';
dlg_title = 'This is the great I-V CURVE CALCULATOR made by Lovinda degrden';
num_lines= 1;
defAns = {'12 8','3','6.46','64.9', '63', '0.002', '1.278', '6.8e-9','25', '0.1'};
%defAns = {'9 4','2','5.82','23.11', '6.84', '0.007', '1.142', '2e-9','25', '0.1'};
answerStr = inputdlg(prompt,dlg_title,num_lines,defAns,options);

answer = [0 str2double(answerStr{2}) str2double(answerStr{3}) str2double(answerStr{4}) ...
    str2double(answerStr{5}) str2double(answerStr{6}) str2double(answerStr{7}) str2double(answerStr{8}) ...
    str2double(answerStr{9}) str2double(answerStr{10})];

rowCol = str2num(answerStr{1});
row = rowCol(1);
col = rowCol(2);
nCell = row*col;
I_step = answer(10);

e=0;

nSubstr=0;

%Check number of cells
if nCell ==0
   e=errordlg('Number of cells cannot be zero!');
   waitfor(e);  % code execution is stopped till e is dismissed.
end
    
%Check number of bypass diodes. If entered zero, a bypass diode will be
%added to the whole module.
if answer(2) == 0
   nBp = 1;
elseif answer(2) < 0
   e=errordlg('Number of bypass diodes cannot be negative!'); %Playing with error messages
   waitfor(e); 
else
   nBp = answer(2);
end

end %while

%Resistances
R_sCell = answer(6);
R_pCell = answer(5);

%Estimated cell OC voltage
Voc = answer(4);
Voc_cell = Voc/nCell;
%Diode thermal voltage at 25 degC
V_T25 = 8.6173e-05*298.15; %kT/q [V]

%Diode equation I_D = I_0*(e^(qV_D/AkT)-1). Diode voltage: V_D =
%(kT/q)*ln(I/I_0 +1)
k = 8.617332478e-5; % (Boltzmann constant)
q = 1; %eV
A = answer(7); %ideality factor
T = answer(9)+273.15; %Kelvin, 25 degrees Celcius
I_scSTC = answer(3);
I_0 = answer(8);
V_T=(k*T*A/q); %thermal voltage INCLUDING IDEALITY FACTOR A

V_D_oc = (k*T*A/q)*log((I_scSTC/I_0)+1);  %log = natural log

%What to simulate next. No shading/varying irradiance OR Partial shading
button = questdlg('Is there any shading on the module?', ...
    'To shade or not to shade -- that is the question', 'NO, there is no shading', 'YES, shading occurs', 'Cancel', 'YES, shading occurs');

if strcmp(button,'NO, there is no shading')
    calcByIcell;
elseif strcmp(button,'YES, shading occurs')
    if nBp == 1 %No substrings. Shading location does not matter 
        shadingNoBypass;
    else %Module has bypass diodes. Run simulations based on where shading is.    
        shadingBypassV2;
    end
else
    return %stops script (Cancel)
end