prompt = {'Enter effective irradiance for the module'};
options.Resize='on';
dlg_title = 'Irradiance';
num_lines= 1;
defAns = {'1000'};
answerStr = inputdlg(prompt,dlg_title,num_lines,defAns,options);
E_eff = str2double(answerStr{1});

%Resistances
R_sCell = answer(6);
R_pCell = answer(5);

%Estimated cell OC voltage
Voc = answer(4);
Voc_cell = Voc/nCell;

V_D_oc = V_T*log((I_scSTC/I_0)+1);  %log = natural log

%Using irradiance to estimate short-circuit current
I_sc1 = (E_eff/1000)*I_scSTC; %~= I_PH


%% Make vectors for calculation of I-V characteristic
I_cell(1) = 0;
I_step = 0.05;
V_D(1) = V_T*log((I_sc1/I_0)+1);
V_cell(1) = V_D(1);

i = 1;
syms x
while I_cell < I_sc1
    I_cell(i+1) = I_cell(i) + I_step;
    V_D(i+1) = solve(I_cell(i+1) - I_sc1 + I_0*(exp(x/V_T)-1) + x/R_pCell == 0, x);
    V_cell(i+1) = V_D(i+1) - I_cell(i+1)*R_sCell;

    i = i+1;
end

I_cell_1 = fliplr(I_cell);
V_cell_1 = fliplr(V_cell);

P_cell = V_cell_1.*I_cell_1; %cell power!
V_mod = V_cell_1*nCell;
P_mod = V_mod.*I_cell_1;

%% Making nice plots
figure(1)
[ax,p1,p2] = plotyy(V_mod,I_cell_1,V_mod,P_mod);

% Add a title and axis labels
title('PV module I-V characteristic')
xlabel(ax(1),'V_{module}') % label x-axis
ylabel(ax(1),'I_{module}') % label left y-axis
ylabel(ax(2),'Module power') % label right y-axis
legend('Module I-V curve','Module power curve','Location','southoutside','Orientation','horizontal')

set(ax(1),'YLim',[0 inf])
set(ax(2),'YLim',[0 inf])
set(ax(1),'YTick',[0:1:10])
grid on

figure(2)
plot(V_cell_1*1, I_cell_1, V_cell_1*2, I_cell_1, V_cell_1*3, I_cell_1);

title('PV cell I-V characteristic for 1, 2 and 3 cells')
xlabel('V_{cell(s)}') % label x-axis
ylabel('I_{cell}') % label left y-axis
legend('1 cell','2 cells in series', '3 cells in series','Location','southoutside','Orientation','horizontal')
set(gca, 'YLim', [0, get(gca, 'YLim') * [0; 1]])
set(gca, 'XLim', [0, get(gca, 'XLim') * [0; 1]])
grid on