%i = step, j = shaded cells per substring, ss = substrings

prompt = {'Number of cells in each substring:', ... %input 1
    'Number of shaded cells on each substring:', ... %input 2
    'Specify percentage [%] of shading for each shaded cell, separated by space:', ... %input 3
    'Solar cell reverse breakdown voltage (NB! Negative value) [V]:', ... %input 4
    'Solar irradiance [W/m2]:', ... %input 5
    'Bypass diode forward voltage [V]:'}; %input 6

err=1;
while err ~=0
    options.Resize='on';
    dlg_title = 'Input shading conditions';
    num_lines= 1;
    defAns = {'24 48 24','1 0 0','50', '-5.5', '800', '0.3'};
    %defAns = {'18 18','2 0','70 50', '-25', '1000', '0.3'};

    shadingStr = inputdlg(prompt,dlg_title,num_lines,defAns,options);
    
    err=0;
    
    nCellsSubstr = str2num(shadingStr{1});
    nBp = length(nCellsSubstr);
    nCellShaded = str2num(shadingStr{2});
    shading = str2num(shadingStr{3});
    
    V_BDcell = str2double(shadingStr{4}); %reverse breakdown voltage
    
    if nCellShaded ~= length(shading) %sjekk om input er riktig.
        err=errordlg('Specified shading does not match the number of shaded cells!');
        waitfor(err);
    end
    if V_BDcell >= 0 %sjekk om input er riktig.
        err=errordlg('Reverse breakdown voltage must be less than zero!');
        waitfor(err);
    end    
    
end %while, input. Checking for errors

Vfwd_bp = str2double(shadingStr{6});
Abp = 1.05; %for schottky diode
I0_bp = 0.7e-9; 

E_eff = str2double(shadingStr{5});
%Using irradiance to estimate short-circuit current for non-shaded cells,
%disregarding temperature effects
I_step = answer(10);
I_scNS = (E_eff/1000)*I_scSTC; %~= I_PH

I_PV = 0:I_step:(I_scNS+I_step); %current vector
vectorSize = length(I_PV);

%Allocating for speed
I_str = zeros(nBp, vectorSize); %current for each string
V_str = zeros(nBp, vectorSize); %voltage for each substring
I_bp = zeros(nBp, vectorSize);

%non-shaded cells
V_D_NS = zeros(nBp, vectorSize);
V_cellNS = zeros(nBp, vectorSize);
V_T=(k*T*A/q); %thermal voltage

I_cellSHsc = zeros(nBp, max(nCellShaded)); %row = substring, col = cell number in substring
V_D_SH = zeros(max(nCellShaded), vectorSize, nBp);  %3 dimensional. row = cell number, col = step, page = substring
V_cellSH = zeros(max(nCellShaded), vectorSize, nBp);
nCellNS = zeros(1,nBp);

syms x
for ss = 1:1:nBp %going through the substrings (ss) Initializing
    V_D_NS(ss,1) = V_T*log((I_scNS/I_0)+1);
    V_cellNS(ss,1) = V_D_NS(ss,1);
    nCellNS(ss) = nCellsSubstr(ss)-nCellShaded(ss); 
     %number of non-shaded cells per substring
    V_str(ss,1) = V_cellNS(ss,1)*nCellNS(ss); %substring with non-shaded cells' OC voltages.
    
    if nCellShaded(ss) > 0 %
        for j=1:1:nCellShaded(ss)
            I_cellSHsc(ss,j) = (1-(shading(j)/100))*I_scNS; %reduced sc current in shaded cell(s)
            V_D_SH(j,1,ss) = V_T*log((I_cellSHsc(ss,j)/I_0)+1); %initial values shaded cells, at OPEN CIRCUIT
            V_cellSH(j,1,ss) = V_D_SH(j,1,ss); % - Rs*Ipv=0
            V_str(ss,1) = V_str(ss,1) + V_cellSH(j,1,ss); %add all the shaded cells' OC voltages
        end
    end
end

for i = 2:1:vectorSize %stepping through I_PV
    %Variables: I_PV(i), I_str(ss,i), V_str(ss,i), 
    %V_D_NS(i), V_cellNS(i), V_cellSH(j,i,ss)
    
    for ss = 1:1:nBp %going through the substrings (ss)
    %non-shaded cells
    V_D_NS(ss,i) = vpasolve(I_PV(i) - I_scNS + I_0*(exp(x/V_T)-1) + x/R_pCell == 0, x, V_D_NS(ss,i-1));
    V_cellNS(ss,i) = V_D_NS(ss,i) - I_PV(i)*R_sCell;
      
      V_str(ss,i) = V_cellNS(ss,i)*nCellNS(ss);
      I_str(ss,i) = I_PV(i); %current equal to module current unless bypass diode activates
    
      %for shaded cells
      if nCellShaded(ss) > 0 % 
        for j=1:1:nCellShaded(ss)
            if I_str(ss,i) > I_cellSHsc(ss,j) %Check if cell is in reverse bias
                V_D_SH(j,i,ss) = (I_cellSHsc(ss,j) - I_str(ss,i))*R_pCell;
                
                %ASSUME BD WILL NOT HAPPEN WHERE THERE IS BYPASS DIODES
                %BUT HERE IS SOMETHING FOR MAKING SURE THERE IS NOT TOO
                %MANY CELLS IN A SUBSTRING
                if V_D_SH(j,i,ss) <= V_BDcell %chech if reverse breakdown voltage is reached
                    V_D_SH(j,i,ss) = V_BDcell; %simplified
                    %I_D_SH(j,i+1) = KARAKTERISTIKK MED BD
                    % if I_D_SH(j,i+1) >= (I_cell(i+1) - I_cellSHsc(j,1))
                    %I_D_SH(j,i) = (I_PV(i) - I_cellSHsc(ss,j));
                % end
                % else
                    %I_D_SH(j,i+1) = 0; %diode is blocking in reverse voltage. No BD.
                end
                V_cellSH(j,i,ss) = V_D_SH(j,i,ss) - I_str(ss,i)*R_sCell;
            
            else %not in reverse bias
                V_D_SH(j,i,ss) = vpasolve(I_str(ss,i) - I_cellSHsc(ss,j) + I_0*(exp(x/V_T)-1) + x/R_pCell == 0,x,V_cellSH(j,i-1,ss));
                    %I_D_SH(j,i) = I_0*(exp(V_D_SH(j,i)/V_T)-1);
                V_cellSH(j,i,ss) = V_D_SH(j,i,ss) - I_str(ss,i)*R_sCell;
            end %if 
        V_str(ss,i) = V_str(ss,i) + V_cellSH(j,i,ss); %adding the shaded voltages
        end %for nCellShaded
       
        if V_str(ss,i) <= 0 %check if substr voltage is negative. If not, carry on
            I_bp(ss,i) = I0_bp*(exp(-V_str(ss,i)/(V_T*(Abp/A)))-1);
            if I_bp(ss,i) > I_PV(i) %current cannot be larger than PV current
                I_bp(ss,i) = I_PV(i);
            end
            
            if V_str(ss,i) <= -Vfwd_bp %bypass diode limits voltage
                V_str(ss,i) = -Vfwd_bp;
            end
            I_str(ss,i) = I_str(ss,i) - I_bp(ss,i);
        
            %Calculate new voltages for substring's cells with reduced
            %current
            %non-shaded cells
            V_D_NS(ss,i) = vpasolve(I_str(ss,i) - I_scNS + I_0*(exp(x/V_T)-1) + x/R_pCell == 0, x, V_D_NS(ss,i-1));
            V_cellNS(ss,i) = V_D_NS(ss,i) - I_str(ss,i)*R_sCell; %OBS!! Kan finne p  bli RADvektor!

            %for shaded cells
            if nCellShaded(ss) > 0 % 
                for j=1:1:nCellShaded(ss)
                    if I_str(ss,i) > I_cellSHsc(ss,j) %Check if cell is in reverse bias WHICH IT IS
                        V_D_SH(j,i,ss) = (I_cellSHsc(ss,j) - I_str(ss,i))*R_pCell;
                        V_cellSH(j,i,ss) = V_cellSH(j,i,ss) - I_str(ss,i)*R_sCell; %no BD yet
            
                    elseif I_str(ss,i) <= I_cellSHsc(ss,j) %not in reverse bias
                        V_D_SH(j,i,ss) = vpasolve(I_str(ss,i) - I_cellSHsc(ss,j) + I_0*(exp(x/V_T)-1) + x/R_pCell == 0,x,V_cellSH(j,i-1,ss));
                        V_cellSH(j,i,ss) = V_D_SH(j,i,ss) - I_str(ss,i)*R_sCell;
                    end %if 
                    
                end %for nCellShaded
            end %if negative substr voltage
        end %if V_ss neg
      end %if ss > 0
    end %for ss =1:1:nBp
     i = i+1;    
end %for i

%no shading
I_cellNS = fliplr(I_PV);
V_cellNS1 = fliplr(V_cellNS(2,:));
P_cellNS = V_cellNS1.*I_cellNS;
V_mod1 = zeros(1, vectorSize);

%shaded cells
V_cellSH1 = zeros(max(nCellShaded), vectorSize, nBp);
P_cellSH = zeros(max(nCellShaded), vectorSize, nBp);  %allocating for speed
I_str1 = zeros(nBp, vectorSize);
for ss = 1:1:nBp
    I_str1(ss,:)=fliplr(I_str(ss,:));
    V_mod1 = V_mod1 + V_str(ss,:);
    for j=1:1:nCellShaded 
        V_cellSH1(j,:,ss) = fliplr(V_cellSH(j,:,ss));
        P_cellSH(j,:,ss) = V_cellSH1(j,:,ss).*I_str1(ss,:);
    end
end

V_PV=fliplr(V_mod1);
I_PV1=fliplr(I_PV);

P_mod=V_PV.*I_PV1;

%PLOT
figure(1)
[ax,q1,q2] = plotyy(V_PV,I_PV1,V_PV,P_mod);
% Add a title and axis labels
title('PV module I-V characteristic')
xlabel(ax(1),'V_{module}') % label x-axis
ylabel(ax(1),'I_{module}') % label left y-axis
ylabel(ax(2),'Module power') % label right y-axis
legend('Module I-V curve','Module power curve','Location','southoutside','Orientation','horizontal')
P_max = roundn(max(P_mod),1)+20;
P_min = roundn(min(P_mod),1)-10;
set(ax(2),'YLim',[P_min inf])
set(ax(1),'YLim',[0 inf])
set(ax(1),'YTick',[0:0.5:(ceil(I_scNS)+1)])
set(ax(2),'YTick',[P_min:50:P_max])
set(ax,{'ycolor'},{'r';'b'})  % Left color red, right color blue
grid on

figure(2)
[ax,p1,p2] = plotyy(V_cellNS1,I_cellNS,V_cellNS1,P_cellNS);
title('I-V and power characteristics for unshaded cell')
xlabel(ax(1),'V_{cell}') % label x-axis
ylabel(ax(1),'I_{module}') % label left y-axis
ylabel(ax(2),'Cell power') % label right y-axis
legend('Unshaded cell I-V curve','Unshaded cell power curve','Location','southoutside','Orientation','horizontal')
P_maxNS = roundn(max(P_cellNS),1)+5;
P_minNS = roundn(min(P_cellNS),1)-5;
set(ax(2),'YTick',[P_minNS:1:P_maxNS])
set(ax(1),'YTick',[0:1:(ceil(I_scNS)+1)])
set(ax(1),'YLim',[0 inf])
set(ax(2),'YLim',[0 inf])
grid on