prompt = {'Number of shaded cells:', ... %input 1
    'Specify percentage [%] of shading for each shaded cell, separated by space:', ... %input 2
    'Solar cell reverse breakdown voltage (NB! Negative value) [V]:', ... %input 3
    'Solar irradiance [W/m2]:'}; %input 4

e=1;
while e ~=0
    options.Resize='on';
    dlg_title = 'Input shading conditions';
    num_lines= 1;
    defAns = {'1','75', '-5.5', '800'};
    shadingStr = inputdlg(prompt,dlg_title,num_lines,defAns,options);
    
    e=0;
    
    V_BDcell = str2double(shadingStr{3});
    nCellShaded = str2double(shadingStr{1});
    shading = str2num(shadingStr{2});
    V_rbd = str2double(shadingStr{3}); %reverse breakdown voltage
    if nCellShaded ~= length(shading) %sjekk om input er riktig.
        e=errordlg('Specified shading does not match the number of shaded cells!');
        waitfor(e);
    end
    if V_BDcell >= 0 %sjekk om input er riktig.
        e=errordlg('Reverse breakdown voltage must be less than zero!');
        waitfor(e);
    end    
    
end %while, input. check for errors

E_eff = str2double(shadingStr{4});
%Using irradiance to estimate short-circuit current for non-shaded cells
I_scNS = (E_eff/1000)*I_scSTC; %~= I_PH

I_step = answer(10);
vectorSize = floor(I_scNS/I_step)-1;

%Allocating for speed
I_cell = zeros(1, vectorSize);
I_cell(1) = 0;
V_D_NS = zeros(1, vectorSize);
V_cellNS = zeros(1, vectorSize);

V_D_NS(1) = V_T*log((I_scNS/I_0)+1); %open circuit
V_cellNS(1) = V_D_NS(1);

I_cellSHsc = zeros(nCellShaded, vectorSize);
I_D_SH = zeros(nCellShaded, vectorSize);
V_D_SH = zeros(nCellShaded, vectorSize);
V_cellSH = zeros(nCellShaded, vectorSize);

%Shaded cells short-circuit currents
for j=1:1:nCellShaded %initial values shaded cells, at OPEN CIRCUIT
    I_cellSHsc(j,1) = (1-(shading(j)/100))*I_scNS; %reduced sc current in shaded cell(s)
    V_D_SH(j,1) = V_T*log((I_cellSHsc(j,1)/I_0)+1);
    V_cellSH(j,1) = V_D_SH(j,1);
end

nCellNS = nCell-nCellShaded; %number of non-shaded cells

%% Vectors for calculation of I-V characteristic

i = 1;
syms x
while I_cell < (I_scNS+1)
    I_cell(i+1) = I_cell(i) + I_step;
        
    %non-shaded cells
    V_D_NS(i+1) = vpasolve(I_cell(i+1) - I_scNS + I_0*(exp(x/V_T)-1) + x/R_pCell == 0, x, V_D_NS(i));
    V_cellNS(i+1) = V_D_NS(i+1) - I_cell(i+1)*R_sCell; %OBS!! Kan finne p  bli RADvektor!

    %for shaded cells
    for j=1:1:nCellShaded
     
        if I_cell(i+1) > I_cellSHsc(j,1) %Check if cell is in reverse bias
            V_D_SH(j,i+1) = (I_cellSHsc(j,1) - I_cell(i+1))*R_pCell;
            if V_D_SH(j,i+1) <= V_BDcell %chech if reverse breakdown voltage is reached
                V_D_SH(j,i+1) = V_BDcell-0.5;
                 %INSERT DIODE CHARACTERISTIC LATER
                  I_D_SH(j,i+1) = (I_cell(i+1) - I_cellSHsc(j,1));
                % end
            else
                I_D_SH(j,i+1) = 0; %diode is blocking in reverse voltage. No BD.
            end
            V_cellSH(j,i+1) = V_D_SH(j,i+1) - I_cell(i+1)*R_sCell; %no BD yet
            
        else
            V_D_SH(j,i+1) = vpasolve(I_cell(i+1) - I_cellSHsc(j,1) + I_0*(exp(x/V_T)-1) + x/R_pCell == 0, x);
            I_D_SH(j,i+1) = I_0*(exp(V_D_SH(j,i+1)/V_T)-1);
            V_cellSH(j,i+1) = V_D_SH(j,i+1) - I_cell(i+1)*R_sCell;
        end %if
        
    end %for
    
    i = i+1;
end

%no shading
I_cell1 = fliplr(I_cell);
V_cellNS1 = fliplr(V_cellNS);
P_cellNS = V_cellNS1.*I_cell1;

V_mod = V_cellNS1*nCellNS; %Sum voltages for healthy cells first

%shaded cells
V_cellSH1 = zeros(nCellShaded, length(V_cellSH)); %allocating for speed
P_cellSH = zeros(nCellShaded, length(V_cellSH));  %allocating for speed
for j=1:1:nCellShaded 
   V_cellSH1(j,:) = fliplr(V_cellSH(j,:));
   V_mod = V_mod + V_cellSH1(j,:);
   P_cellSH(j,:) = V_cellSH1(j,:).*I_cell1;
end

P_mod = V_mod.*I_cell1;

%% Making plots
for j=1:1:nCellShaded
    figure(j)
    [ax,~,~] = plotyy(V_cellSH1(j,:),I_cell1,V_cellSH1(j,:),P_cellSH(j,:));
    title('I-V and power characteristics for shaded cell')
    P_maxSH = roundn(max(P_cellSH(j,:)),1)+5;
    P_minSH = roundn(min(P_cellSH(j,:)),1)-5;
    set(ax(2),'YTick',[P_minSH:10:P_maxSH])
    set(ax(1),'YLim',[0 inf])
    set(ax(2),'YLim',[P_minSH inf])
    set(ax(1),'YTick',[0:1:(ceil(I_scNS))])
    xlabel(ax(1),'V_{cell}') % label x-axis
    ylabel(ax(1),'I_{module}') % label left y-axis
    ylabel(ax(2),'Power') % label right y-axis
    legend('Shaded cell I-V curve','Shaded cell power curve','Location','southoutside','Orientation','horizontal')
    grid on
end

figure(j+1)
[ax,p1,p2] = plotyy(V_cellNS1,I_cell1,V_cellNS1,P_cellNS);
title('I-V and power characteristics for unshaded cell')
xlabel(ax(1),'V_{cell}') % label x-axis
ylabel(ax(1),'I_{module}') % label left y-axis
ylabel(ax(2),'Cell power') % label right y-axis
legend('Unshaded cell I-V curve','Unshaded cell power curve','Location','southoutside','Orientation','horizontal')
P_maxNS = roundn(max(P_cellNS),1)+5;
P_minNS = roundn(min(P_cellNS),1)-5;
set(ax(2),'YTick',[P_minNS:1:P_maxNS])
set(ax(1),'YTick',[0:1:(ceil(I_scNS)+1)])
set(ax(1),'YLim',[0 inf])
set(ax(2),'YLim',[0 inf])
grid on

figure(j+2)
[ax,q1,q2] = plotyy(V_mod,I_cell1,V_mod,P_mod);
% Add a title and axis labels
title('PV module I-V characteristic')
xlabel(ax(1),'V_{module}') % label x-axis
ylabel(ax(1),'I_{module}') % label left y-axis
ylabel(ax(2),'Module power') % label right y-axis
legend('Module I-V curve','Module power curve','Location','southoutside','Orientation','horizontal')
P_max = roundn(max(P_mod),1)+10;
P_min = roundn(min(P_mod),1)-10;
set(ax(2),'YLim',[P_min inf])
set(ax(1),'YLim',[0 inf])
set(ax(1),'YTick',[0:1:(ceil(I_scNS)+1)])
set(ax(2),'YTick',[P_min:100:P_max])
grid on